data{
  // scalars
  int<lower=0> nCond;
  int<lower=0> nCov;
  int<lower=0> nLab;
  int<lower=0> nSubj;
  int<lower=0> nRow;
  int<lower=0> nDepvar;
  
  // continuous data
  real y[nRow];
  int<lower=1,upper=nSubj> subj[nRow];
  int<lower=1,upper=nLab> lab[nRow];
  int<lower=1,upper=nDepvar> depvar[nRow];
  int<lower=1,upper=nCond> condition[nRow];
  real<lower=0> dv_scale[nDepvar];
  
  // covariates  
  matrix[nRow,nCov] XX;
}

transformed data{
  int nFac = 3;
}

parameters{
  real alpha_int_cd[nCond,nDepvar];
  vector[nCov] alpha_XX;
  real<lower=0> sigma_d[nDepvar];
  
  vector[nFac] phi_beta_lf[nLab];
  vector<lower=0>[nFac] omega_phi_beta_f;
  corr_matrix[nFac] Rho_phi_beta_ff;
  real lambda_beta1_d[nDepvar];
  real lambda_beta2_cd[nCond,nDepvar];
  
  vector[nCond] eta_ldc[nLab,nDepvar];
  vector<lower=0>[nCond] omega_eta_dc[nDepvar];
  corr_matrix[nCond] Rho_eta_dcc[nDepvar];

  vector[nFac-1+nCov] phi_gamma_sf[nSubj];
  vector<lower=0>[nFac-1+nCov] omega_phi_gamma_f;
  corr_matrix[nFac-1+nCov] Rho_phi_gamma_ff;
  real lambda_gamma1_d[nDepvar];
}

transformed parameters{    
  real mu[nRow];
  real<lower=0> sig[nRow];
  
  real beta_lcd[nLab,nCond,nDepvar];  
  cov_matrix[nFac] Omega_phi_beta_ff = quad_form_diag(Rho_phi_beta_ff, omega_phi_beta_f);
  cov_matrix[nFac-1+nCov] Omega_phi_gamma_ff = quad_form_diag(Rho_phi_gamma_ff, omega_phi_gamma_f);
  cov_matrix[nCond] Omega_eta_dcc[nDepvar];  
  for(i in 1:nDepvar){
    Omega_eta_dcc[i] = quad_form_diag(Rho_eta_dcc[i], omega_eta_dc[i]);
  }
  
  for(i in 1:nLab){
    for(j in 1:nCond){
	  for(k in 1:nDepvar){
	    beta_lcd[i,j,k] = phi_beta_lf[i,1] + 
							phi_beta_lf[i,2] * lambda_beta1_d[k] + 
							phi_beta_lf[i,3] * lambda_beta2_cd[j,k] + 
							eta_ldc[i,k,j];
	  }
	}
  }
  
  for(i in 1:nRow){
	sig[i] = sigma_d[depvar[i]];
    mu[i] = alpha_int_cd[condition[i],depvar[i]] + 
					XX[i] * alpha_XX +
					beta_lcd[lab[i],condition[i],depvar[i]] +
					phi_gamma_sf[subj[i],1] +
					phi_gamma_sf[subj[i],2] * lambda_gamma1_d[depvar[i]] +
					XX[i] * phi_gamma_sf[subj[i],3:(nFac-1+nCov)];
  }
}

model{
  for(i in 1:nCond){
    alpha_int_cd[i] ~ normal(0, 50);
  }
  alpha_XX ~ normal(0, 5);
  sigma_d ~ cauchy(0, dv_scale);
    
  for(i in 1:nLab){
    phi_beta_lf[i] ~ multi_normal(rep_vector(0, nFac), Omega_phi_beta_ff);
  }
  omega_phi_beta_f ~ cauchy(0, 2.5);
  Rho_phi_beta_ff ~ lkj_corr(1);
  lambda_beta1_d ~ normal(0, 1);
  for(i in 1:nCond){
  	lambda_beta2_cd[i] ~ normal(0, 1);
  }
  
  for(i in 1:nDepvar){
  	eta_ldc[:,i] ~ multi_normal(rep_vector(0, nCond), Omega_eta_dcc[i]);
	omega_eta_dc[i] ~ cauchy(0, 2.5);
	Rho_eta_dcc[i] ~ lkj_corr(1);
  }
  
  phi_gamma_sf ~ multi_normal(rep_vector(0, nFac-1+nCov), Omega_phi_gamma_ff);
  omega_phi_gamma_f ~ cauchy(0, 2.5);
  Rho_phi_gamma_ff ~ lkj_corr(1);
  lambda_gamma1_d ~ normal(0, 1);
  
  y ~ normal(mu, sig);
}

generated quantities{
  real log_lik = normal_lpdf(y | mu, sig);
}
