data{
  // scalars
  int<lower=0> nCond;
  int<lower=2> nCatO;
  int<lower=0> nLab;
  int<lower=0> nSubj;
  int<lower=0> nRowC;
  int<lower=0> nRowB;
  int<lower=0> nRowO;
  int<lower=0> nDepvarC;
  int<lower=0> nDepvarB;
  int<lower=0> nDepvarO;
  
  // continuous data
  real yC[nRowC];
  int<lower=1,upper=nSubj> subjC[nRowC];
  int<lower=1,upper=nLab> labC[nRowC];
  int<lower=1,upper=nDepvarC> depvarC[nRowC];
  int<lower=1,upper=nCond> conditionC[nRowC];
  real<lower=0> dv_scale[nDepvarC];
           
  // binary data
  int<lower=0,upper=1> yB[nRowB];
  int<lower=1,upper=nSubj> subjB[nRowB];
  int<lower=1,upper=nLab> labB[nRowB];
  int<lower=nDepvarC+1,upper=nDepvarC+nDepvarB> depvarB[nRowB];
  int<lower=1,upper=nCond> conditionB[nRowB];

  // ordinal data
  int<lower=1,upper=nCatO> yO[nRowO];
  int<lower=1,upper=nSubj> subjO[nRowO];
  int<lower=1,upper=nLab> labO[nRowO];
  int<lower=nDepvarC+nDepvarB+1,upper=nDepvarC+nDepvarB+nDepvarO> depvarO[nRowO];
  int<lower=1,upper=nCond> conditionO[nRowO];

  // covariates  
  int<lower=0,upper=1> Anch_ind[nDepvarC + nDepvarB + nDepvarO];
}

transformed data{
  int nDepvar = nDepvarC + nDepvarB + nDepvarO;
  int nFac_beta = 7;
  int nFac_gamma = 5;
  int nAnch = sum(Anch_ind);
  
  int<lower=1,upper=nDepvarO> depvarO_base[nRowO] = depvarO;
  for(i in 1:nRowO){
    depvarO_base[i] = depvarO[i] - nDepvarC - nDepvarB;
  }
}

parameters{
  real alpha_cd_rawCB[nCond,nDepvarC+nDepvarB];
  real alpha_cd_rawO[nCond-1,nDepvarO];
  real<lower=0> sigma_dC[nDepvarC];
  ordered[nCatO-1] cutpoints_dO[nDepvarO];
  
  vector[nFac_beta] phi_beta_lf[nLab];
  vector<lower=0>[nFac_beta] omega_phi_beta_f;
  corr_matrix[nFac_beta] Rho_phi_beta_ff;
  real lambda_beta1_d[nDepvar];
  real lambda_beta2_cd[nCond,nDepvar];
  real lambda_beta3_d_raw[nAnch];
  real lambda_beta4_cd_raw[nCond,nAnch];
  
  vector[nCond] eta_ldc[nLab,nDepvar];
  vector<lower=0>[nCond] omega_eta_dc[nDepvar];
  corr_matrix[nCond] Rho_eta_dcc[nDepvar];

  vector[nFac_gamma] phi_gamma_sf[nSubj];
  vector<lower=0>[nFac_gamma] omega_phi_gamma_f;
  corr_matrix[nFac_gamma] Rho_phi_gamma_ff;
  real lambda_gamma1_d[nDepvar];
  real lambda_gamma2_cd[nCond,nDepvar];
  real lambda_gamma3_cd_raw[nCond,nAnch];
}

transformed parameters{
  real alpha_cd[nCond,nDepvar];
  real lambda_beta3_d[nDepvar];
  real lambda_beta4_cd[nCond,nDepvar];
  real lambda_gamma3_cd[nCond,nDepvar];
    
  real beta_lcd[nLab,nCond,nDepvar];  
  cov_matrix[nFac_beta] Omega_phi_beta_ff = quad_form_diag(Rho_phi_beta_ff, omega_phi_beta_f);
  cov_matrix[nFac_gamma] Omega_phi_gamma_ff = quad_form_diag(Rho_phi_gamma_ff, omega_phi_gamma_f);
  cov_matrix[nCond] Omega_eta_dcc[nDepvar];

  real muC[nRowC];
  real<lower=0> sigC[nRowC];
  real muB[nRowB];
  vector[nRowO] muO;
  
  for(i in 1:nCond){
    for(j in 1:(nDepvarC+nDepvarB)){
	  alpha_cd[i,j] = alpha_cd_rawCB[i,j];
	}
  }
  for(i in 1:nDepvarO){
    alpha_cd[1,nDepvarC+nDepvarB+i] = 0;
  }
  for(i in 2:nCond){
    for(j in 1:nDepvarO){
      alpha_cd[i,nDepvarC+nDepvarB+j] = alpha_cd_rawO[i-1,j];
	}
  }
  
  for(i in 1:nAnch){
  	lambda_beta3_d[i] = lambda_beta3_d_raw[i];
	for(j in 1:nCond){
  		lambda_beta4_cd[j,i] = lambda_beta4_cd_raw[j,i];
  		lambda_gamma3_cd[j,i] = lambda_gamma3_cd_raw[j,i];
	}
  }
  for(i in (nAnch+1):nDepvar){
  	lambda_beta3_d[i] = 0;
	for(j in 1:nCond){
  		lambda_beta4_cd[j,i] = 0;
  		lambda_gamma3_cd[j,i] = 0;
	}
  }
  
  for(i in 1:nDepvar){
    Omega_eta_dcc[i] = quad_form_diag(Rho_eta_dcc[i], omega_eta_dc[i]);
  }
  
  for(i in 1:nLab){
    for(j in 1:nCond){
	  for(k in 1:nDepvar){
	    beta_lcd[i,j,k] = phi_beta_lf[i,1] + 
							phi_beta_lf[i,2] * lambda_beta1_d[k] + 
							phi_beta_lf[i,3] * lambda_beta2_cd[j,k] + 
							phi_beta_lf[i,4] * (j==1) + 
							phi_beta_lf[i,5] * (j==2) + 
							phi_beta_lf[i,6] * lambda_beta3_d[k] + 
							phi_beta_lf[i,7] * lambda_beta4_cd[j,k] + 
							eta_ldc[i,k,j];
	  }
	}
  }
  
  for(i in 1:nRowC){
	sigC[i] = sigma_dC[depvarC[i]];
    muC[i] = sigma_dC[depvarC[i]] * 
				(alpha_cd[conditionC[i],depvarC[i]] + 
					beta_lcd[labC[i],conditionC[i],depvarC[i]] +
					phi_gamma_sf[subjC[i],1] +
					phi_gamma_sf[subjC[i],2] * lambda_gamma1_d[depvarC[i]] +
					phi_gamma_sf[subjC[i],3] * lambda_gamma2_cd[conditionC[i],depvarC[i]] +
					phi_gamma_sf[subjC[i],4] * Anch_ind[depvarC[i]] +
					phi_gamma_sf[subjC[i],5] * lambda_gamma3_cd[conditionC[i],depvarC[i]]);
  }
  
  for(i in 1:nRowB){
    muB[i] = (alpha_cd[conditionB[i],depvarB[i]] + 
					beta_lcd[labB[i],conditionB[i],depvarB[i]] +
					phi_gamma_sf[subjB[i],1] +
					phi_gamma_sf[subjB[i],2] * lambda_gamma1_d[depvarB[i]] +
					phi_gamma_sf[subjB[i],3] * lambda_gamma2_cd[conditionB[i],depvarB[i]]);
  }

  for(i in 1:nRowO){
    muO[i] = (alpha_cd[conditionO[i],depvarO[i]] + 
					beta_lcd[labO[i],conditionO[i],depvarO[i]] +
					phi_gamma_sf[subjO[i],1] +
					phi_gamma_sf[subjO[i],2] * lambda_gamma1_d[depvarO[i]] +
					phi_gamma_sf[subjO[i],3] * lambda_gamma2_cd[conditionO[i],depvarO[i]]);
  }
}

model{
  for(i in 1:nCond){
    alpha_cd_rawCB[i] ~ normal(0, 5);
  }
  for(i in 1:(nCond-1)){
    alpha_cd_rawO[i] ~ normal(0, 5);
  }
  sigma_dC ~ cauchy(0, dv_scale);
  for(i in 1:nDepvarO){
    cutpoints_dO[i] ~ normal(0, 5);
  }
    
  for(i in 1:nLab){
    phi_beta_lf[i] ~ multi_normal(rep_vector(0, nFac_beta), Omega_phi_beta_ff);
  }
  omega_phi_beta_f ~ cauchy(0, 2.5);
  Rho_phi_beta_ff ~ lkj_corr(1);
  lambda_beta1_d ~ normal(0, 1);
  lambda_beta3_d_raw ~ normal(0, 1);
  for(i in 1:nCond){
  	lambda_beta2_cd[i] ~ normal(0, 1);
  	lambda_beta4_cd_raw[i] ~ normal(0, 1);
  }
  
  for(i in 1:nDepvar){
  	eta_ldc[:,i] ~ multi_normal(rep_vector(0, nCond), Omega_eta_dcc[i]);
	omega_eta_dc[i] ~ cauchy(0, 2.5);
	Rho_eta_dcc[i] ~ lkj_corr(1);
  }
  
  phi_gamma_sf ~ multi_normal(rep_vector(0, nFac_gamma), Omega_phi_gamma_ff);
  omega_phi_gamma_f ~ cauchy(0, 2.5);
  Rho_phi_gamma_ff ~ lkj_corr(1);
  lambda_gamma1_d ~ normal(0, 1);
  for(i in 1:nCond){
  	lambda_gamma2_cd[i] ~ normal(0, 1);
  	lambda_gamma3_cd_raw[i] ~ normal(0, 1);
  }
  
  yC ~ normal(muC, sigC);
  yB ~ bernoulli(Phi(muB));
  yO ~ ordered_probit(muO, cutpoints_dO[depvarO_base]);
}

generated quantities{
  real log_lik;
  real log_likO[nRowO];
  for(i in 1:nRowO){
  	log_likO[i] = ordered_probit_lpmf(yO[i] | muO[i], cutpoints_dO[depvarO_base[i]]);
  }  
  log_lik = normal_lpdf(yC | muC, sigC) + 
  			bernoulli_lpmf(yB | Phi(muB)) +
			sum(log_likO);
}
