# Read in data
library(foreign)
library(nloptr)
dat <- data.frame(read.spss("CleanedDataset.sav"))
study <- c(study1="Sunk Costs",
		study2="Gain vs loss framing",
		study3a="Anchoring - Distance to NYC",
		study3b="Anchoring - Chicago",
		study3c="Anchoring - Mt. Everest",
		study3d="Anchoring - Babies Born",
		study4="Retrospective gambler fallacy",
		study5="Low vs high category scales",
		study6="Norm of reciprocity",
		study7="Allowed/Forbidden",
		study8="Quote Attribution",
		study9="Flag Priming",
		study10="Currency Priming",
		study11="Imagined contact",
		study12a="Sex differences in implicit math attitudes")#,
		###study12b="Relations between impl. and expl. math attitudes")

var.type <- c("O", "B", "C", "C", "C", "C", "C",
				"B", "B", "B", "O", "C", "C", "C", "C")

# Organize data
YY <- TT <- matrix(NA, nrow(dat), length(study))
OO <- as.matrix( dat[,c("sunkcostorder", "diseaseforder", "anchorinorder", "gamblerforder",
						"scalesorder", "reciprocorder", "allowedforder", "quoteorder",
						"flagprimorder", "moneypriorder", "imaginedorder", "iatorder" )] )
condition2.name <- condition1.name <- rep(NA, length(study))
lab.name <- dat$sample

# Create lab information
ulab <- sort(unique(as.character(lab.name)))
online <- nonUS <- rep(0,length(ulab))
online[c(5,8,9,13,15,18,24,27,30)] <- 1
nonUS[c(2,3,6,9,10,11,14,23,24,28,30)] <- 1
lab.info <- data.frame(Lab=ulab, Online=online, NonUS=nonUS)

# Code dependent measures
i <- 1
condition2.name[i] <- "paid"
condition1.name[i] <- "free"
YY[,i] <- dat$sunkDV
TT[,i] <- 1*(dat$sunkgroup==condition2.name[i])

i <- 2
condition2.name[i] <- "People will die"
condition1.name[i] <- "People will be saved"
YY[,i] <- 1*(dat$gainlossDV=="chose program with probability outcome")
TT[,i] <- 1*(dat$gainlossgroup==condition2.name[i])

i <- 3
condition2.name[i] <- "highanchor"
condition1.name[i] <- "lowanchor"
YY[,i] <- dat$anchoring1
TT[,i] <- 1*(dat$anch1group==condition2.name[i])

i <- 4
condition2.name[i] <- "highanchor"
condition1.name[i] <- "lowanchor"
YY[,i] <- dat$anchoring2
TT[,i] <- 1*(dat$anch2group==condition2.name[i])

i <- 5
condition2.name[i] <- "highanchor"
condition1.name[i] <- "lowanchor"
YY[,i] <- dat$anchoring3
TT[,i] <- 1*(dat$anch3group==condition2.name[i])

i <- 6
condition2.name[i] <- "highanchor"
condition1.name[i] <- "lowanchor"
YY[,i] <- dat$anchoring4
TT[,i] <- 1*(dat$anch4group==condition2.name[i])

i <- 7
condition2.name[i] <- "three6"
condition1.name[i] <- "two6"
YY[,i] <- dat$gambfalDV^2
TT[,i] <- 1*(dat$gambfalgroup==condition2.name[i])

i <- 8
condition2.name[i] <- "high category scale"
condition1.name[i] <- "low category scale"
YY[,i] <- 1*(dat$scales=="more than 2 1/2 hrs")
TT[,i] <- 1*(dat$scalesgroup==condition2.name[i])

i <- 9
condition2.name[i] <- "Asked second"
condition1.name[i] <- "Asked first"
YY[,i] <- 1*(dat$reciprocityus=="yes")
TT[,i] <- 1*(dat$reciprocitygroup==condition2.name[i])

i <- 10
condition2.name[i] <- "allowed"
condition1.name[i] <- "forbidden"
YY[,i] <- 1*( (dat$allowedforbiddenGroup==condition2.name[i] & dat$allowedforbidden=="NO") | 
              (dat$allowedforbiddenGroup==condition1.name[i] & dat$allowedforbidden=="YES") )
TT[,i] <- 1*(dat$allowedforbiddenGroup==condition2.name[i])

i <- 11
condition2.name[i] <- "liked source"
condition1.name[i] <- "disliked source"
YY[,i] <- dat$quote
TT[,i] <- 1*(dat$quoteGroup==condition2.name[i])

i <- 12
condition2.name[i] <- "flag prime"
condition1.name[i] <- "no prime"
YY[,i] <- dat$flagdv
TT[dat$flagGroup==condition2.name[i],i] <- 1
TT[dat$flagGroup==condition1.name[i],i] <- 0

i <- 13
condition2.name[i] <- "Money priming group"
condition1.name[i] <- "Control group"
YY[,i] <- dat$Sysjust
TT[,i] <- 1*(dat$MoneyGroup==condition2.name[i])

i <- 14
condition2.name[i] <- "Contact group"
condition1.name[i] <- "Control group"
YY[,i] <- dat$Imagineddv
TT[,i] <- 1*(dat$ContactGroup==condition2.name[i])

i <- 15
condition2.name[i] <- "female"
condition1.name[i] <- "male"
YY[,i] <- dat$d_art
TT[dat$partgender==condition2.name[i],i] <- 1
TT[dat$partgender==condition1.name[i],i] <- 0

# Set YY and TT to NA if either is NA
sel <- is.na(YY) | is.na(TT)
YY[sel] <- TT[sel] <- NA

# Apply two-parameter Box-Cox transformation to continuous data
gmean <- function(x, na.rm=TRUE){ exp(mean(log(x), na.rm=na.rm)) }
box.cox <- function(y, lambda=c(0,0)){
	if(lambda[1]==0){ log(y + lambda[2])
	}else{ ((y + lambda[2])^lambda[1] - 1) / lambda[1] }
}
nll.bc <- function(lambda, y, X){
	tmpy <- box.cox(y, lambda)
	tmpz <- tmpy / gmean(y+lambda[2])^(lambda[1]-1)
	tmpm <- lm(tmpz ~ X - 1)	
	return(-as.numeric(logLik(tmpm)))
}
nll.bc1 <- function(lambda1,lambda2=0,y,X){ nll.bc(c(lambda1,lambda2),y,X) }
nll.bc2 <- function(lambda2,lambda1=1,y,X){ nll.bc(c(lambda1,lambda2),y,X) }
res.bc <- function(lambda, y, X){
	lambda1 <- lambda[1]
	lambda2 <- lambda[2] 
	tmpy <- box.cox(y, c(lambda1,lambda2))
	tmpm <- lm(tmpy ~ X - 1)
	return(tmpm$residuals)
}

YY0 <- YY
for(i in 1:ncol(YY)){
	if(var.type[i]=="C"){
		sel <- !is.na(YY[,i])		
		tmp.dat <- data.frame(yy=YY[sel,i], ll=as.character(lab.name[sel]), tt=factor(TT[sel,i]))
		tmp.model <- lm(yy ~ ll*tt - ll - tt - 1, data=tmp.dat)
		yy <- tmp.dat$yy
		XX <- model.matrix(tmp.model)
		lambda2.null <- ifelse(all(yy>0), 0, -min(yy)+1E-04)
		lbound <- c(-3, -min(yy)+1E-04)
		ubound <- c(3, 5*max(yy))
		oo <- bobyqa(c(1,lambda2.null), nll.bc, y=yy, X=XX, lower=lbound, upper=ubound)			
		YY[sel,i] <- box.cox(yy, oo$par)
	}
}

# Sign alignment (also relabel conditions to maintain the condition2 > condition1 convention)
RR <- cor(YY, use="pair")
tmp0 <- tmp1 <- YY
tmp0[TT==0] <- NA
tmp1[TT==1] <- NA
tmp.idx <- rep(1:ncol(YY), each=2) + rep(c(0,ncol(YY)), ncol(YY))
tmp2 <- cbind(tmp0,tmp1)[,tmp.idx]
RRR <- cor(tmp2, use="pair")
RRR[is.na(RRR)] <- 0
data.frame(paste0(rep(study,each=2), "*", rep(1:2,ncol(YY))),
	  	   eigen(RRR)$vectors[,1], rep(eigen(RR)$vectors[,1], each=2))

reverse.code <- c(0,1,  1,1,1,1,  0,0,1,0,1,0,0,1,1)

TT0 <- TT
condition2.name0 <- condition2.name
condition1.name0 <- condition1.name
for(i in which(reverse.code==1)){ 
	TT[,i] <- 1 - TT[,i]
	condition2.name[i] <- condition1.name0[i]
	condition1.name[i] <- condition2.name0[i]

	if(var.type[i]=="B"){ YY[,i] <- 1 - YY[,i] }
	if(var.type[i]=="C"){ YY[,i] <- -YY[,i] }
	if(var.type[i]=="O"){ YY[,i] <- (max(YY[,i], na.rm=TRUE) + 1) - YY[,i] }
}

# Write output
study.info <- data.frame(Study=study, VarType=var.type, Reverse=reverse.code,
	Condition1=condition1.name, Condition2=condition2.name)
processed.data <- list(YY=YY, TT=TT, lab.id=lab.name,
	study.info=study.info, lab.info=lab.info, YY0=YY0,
	political.ideology=dat$politicalid)
saveRDS(processed.data, "manylabs.data.rds")
